//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#include "inst/data_section.hh"

namespace inst
{

  // --------------------------------------------------------------------------
  // Constructor and destructor
  // --------------------------------------------------------------------------

  DataSection::~DataSection()
  {
  }

  // --------------------------------------------------------------------------

  bool
  DataSection::define_label(const misc::unique_string &id)
  {
    Label *label = new inst::Label(id, size());
    labels_.push_back(label);

    return Section::register_label(label);
  }

  // --------------------------------------------------------------------------
  // Print operator
  // --------------------------------------------------------------------------

  void
  DataSection::write(std::ostream& ostr) const
  {
    if (0 < size())
      {
        ostr << "\t" << name_ << std::endl;

        std::vector<const Label*>::const_iterator label_it = labels_.begin();

        int next_labeled_offset = size();
        if (!labels_.empty())
          next_labeled_offset = (*label_it)->get_offset();

        for (int i = 0; i < size(); ++i)
          {
            while (i == next_labeled_offset)
              {
                ostr << *(*label_it) << ':' << std::endl;
                ++label_it;

                if (label_it != labels_.end())
                  next_labeled_offset = (*label_it)->get_offset();
                else // No more labels
                  break;
              }

            ostr << "\t.byte\t0x"
                 << std::hex << static_cast<unsigned>(bytes_[i])
                 << std::endl;
          }

        // Print labels that point right after the last data
        while (label_it != labels_.end())
          {
            ostr << *(*label_it) << ':' << std::endl;
            ++label_it;
          }

        ostr << std::dec;
      }
  }

} // namespace inst
